# -----------------------------------------------------------------------------
# MKDoc::Site::ConfigWriter::MKDocRegistrer
# -----------------------------------------------------------------------------
# Author : Jean-Michel Hiver <jhiver@mkdoc.com>
# Copyright : Copyright (c) 2003 MKDoc Holdings Ltd.
# -----------------------------------------------------------------------------
package MKDoc::Site::ConfigWriter::MKDocRegistrer;
use strict;
use warnings;
use Carp;

use MKDoc::Site::Config;
use base qw /MKDoc::Site::ConfigWriter/;

sub class
{
    my $class = shift;
    return ref $class || $class;
}

##
# $class->setup();
# ----------------
# Writes the site.conf and httpd.conf and files.
##
sub setup
{
    my $class = shift;
    my $cfg = new MKDoc::Site::Config;
    
    my $mkdoc_dir = $cfg->{MKDOC_DIR};
    my $site_dir  = $cfg->{SITE_DIR};
    
    my $site_conf_file = File::Spec->canonpath ($mkdoc_dir . '/conf/sites.conf');
    my $http_conf_file = File::Spec->canonpath ($mkdoc_dir . '/conf/httpd.conf');
    my $site_conf = undef;
    
    if (-e $site_conf_file)
    {
	open FP, "<$site_conf_file" || confess "Cannot read-open $site_conf_file: Reason $@";
	my $data = join '', <FP>;
	close FP;
	
	my $VAR1 = undef;
	eval "$data";
	confess $@ if ($@);
	$site_conf = $VAR1;
    }
    
    my $public_domain = $class->site_public_domain();
    my $user_domain   = $class->site_user_domain();
    
    $site_conf->{$public_domain} = $cfg->{SITE_DIR};
    $site_conf->{$user_domain}   = $cfg->{SITE_DIR};
    
    my %unique = map { $_ => 1 } values %{$site_conf};
    my @dirs = keys %unique;
    open FP, ">$http_conf_file" or confess "Cannot write-open $http_conf_file. Reason: $!";
    print FP join "\n", map { "Include $_/httpd.conf" } @dirs;
    close FP;
    
    open FP, ">$site_conf_file" or confess "Cannot write-open $site_conf_file. Reason: $!";
    use Data::Dumper;
    print FP Dumper ($site_conf);
    close FP;
}


1;
