# -------------------------------------------------------------------------------------
# flo::editor::Text
# -------------------------------------------------------------------------------------
# Author : Jean-Michel Hiver <jhiver@mkdoc.com>.
# Copyright : (c) MKDoc Holdings Ltd, 2003
#
# Text content component. Should let the user add simple text to pages
# -------------------------------------------------------------------------------------
package flo::editor::Text;
use flo::Editor;
use flo::Standard;
use MKDoc::CGI;
use MKDoc::Util::Text2HTML;
use MKDoc::XML::XHTML::Linkifier;
use flo::XHTMLFilter;
use Unicode::Normalize;
use lib::util::Command;
use strict;
use Carp;

use flo::Component;
use base qw /flo::Component
	     flo::editor::Mixin::compute_name
	     flo::editor::Mixin::normalize_name/;


sub preferred_extension { 'txt' }


sub new
{
    my $class = shift;
    $class    = ref $class || $class;
    my $self  = bless { @_ }, $class;
    
    my $cgi = $self->{cgi};
    my $param_name = $self->{param_name};
    if (defined $cgi)
    {
	if (defined $cgi->param ($param_name . "_add"))
	{
	    $self->{data} = "";
	}
	else
	{
	    $self->{data} = $cgi->param ($param_name);
	}
    }
    else
    {
	$self->{data} = "";
    }
    
    return $self;
}


sub html
{
    my $self = shift;
    my $data = $self->{'data'};
    
    $data = MKDoc::Util::Text2HTML::text2xml ($data);
    
    my $array = _text_split ($data);
    _make_h2 ($array);
    _make_h3 ($array);
    _make_li ($array);
    _make_ol ($array);
    _make_strong ($array);
    _make_em ($array);
    _make_p  ($array);
    $data = join "\n\n", @{$array};
    
    $data = "<span>$data</span>";
    $data = NFC ($data);
    
    my $tree = new MKDoc::XML::XHTML::Linkifier ( $data );
    $tree->trim_spaces;
    
    $tree->linkify (flo::Editor->links);
    my $res = $tree->text_nice;
    $res =~ s/^.*?<span>//;
    $res =~ s/<\/span>.*?$//;
    return $res;
}

sub _make_strong
{
    my $array = shift;
    for (@{$array})
    {
        s/\*(\w+)\*/<strong>${1}<\/strong>/g;
    }
}

sub _make_em
{
    my $array = shift;
    for (@{$array})
    {
        s/\_(\w+)\_/<em>${1}<\/em>/g;
    }
}

sub _make_p
{
    my $array = shift;
    for (@{$array})
    {
        /^</ and next;
	$_ = "<p>$_</p>";
    }
}


sub _make_h2
{
    my $array = shift;
    for (@{$array})
    {
        /^</ and next;
        s/\n===+\s*$//gsm and $_ = "<h2>$_</h2>";
    }
}


sub _make_h3
{
    my $array = shift;
    for (@{$array})
    {
        /^</ and next;
        s/\n---+\s*$//gsm and $_ = "<h3>$_</h3>";
    }
}


sub _make_li
{
    my $array = shift;
    for (@{$array})
    {
        /^</ and next;
        /^\*/ or next;
        s/\n\*\s*/<\/li>\n<li>/gsm and do {
            s/\*\s*//;
            s/^/<ul>\n<li>/;
            s/$/<\/li>\n<\/ul>/;
        };
    }
}


sub _make_ol
{
    my $array = shift;
    for (@{$array})
    {
        /^</ and next;
        /^\d+\./ or next;
        s/\n\d+\.\s*/<\/li>\n<li>/gsm and do {
            s/\d+\.\s*//;
            s/^/<ol>\n<li>/;
            s/$/<\/li>\n<\/ol>/;
        };
    }
}


sub _text_split
{
    my $data = shift;
    my @data = split /\n\n/, $data;
    my @res  = map {
        $_ =~ s/^(\s|\n|\r)+//gsm;
        $_ =~ s/(\s|\n|\r)+$//gsm;
        $_;
    } @data;
    
    return \@res;
}


sub parse_xml
{
    my $self = shift;
    my $data = shift;
    if ($data =~ /^<perl>/)
    {
	$self->SUPER::parse_xml ($data);
    }
    else
    {
	$self->{data} = $data;
	$self->{data} = shift;
	$self->{data} =~ s/\&lt\;/</g;
	$self->{data} =~ s/\&gt\;/>/g;
	$self->{data} =~ s/\&amp\;/&/g;
	$self->{data} =~ s/&quot;/\"/g;
    }
}


sub as_xhtml { return "<div>" . shift()->html() . "</div>" }


1;
